@@ -7,7 +7,7 @@
 #'   which can be set to one of the
 #'   following options: "exponential", "Weibull", "log-normal",
 #'   "piecewise exponential", or "model averaging". The model averaging
-#'   uses the \code{exp(-bic)} weighting and combines Weibull and
+#'   uses the \code{exp(-bic/2)} weighting and combines Weibull and
 #'   log-normal models. By default, it is set to "model
 #'   averaging".
 #' @param piecewiseSurvivalTime A vector that specifies the time
@@ -144,28 +144,29 @@ fitEvent <- function(df, event_model = "model averaging",
 
     # maximum likelihood estimates and covariance matrix
     if (J > 1) {
-      fit2 <- list(model = "Piecewise exponential",
-                   theta = log(d/ex),
-                   vtheta = diag(1/d),
-                   bic = -2*sum(-d + d*log(d/ex)) + J*log(n0),
-                   piecewiseSurvivalTime = u)
+      vtheta = diag(1/d)
     } else {
-      fit2 <- list(model = "Piecewise exponential",
-                   theta = log(d/ex),
-                   vtheta = 1/d*diag(1),
-                   bic = -2*sum(-d + d*log(d/ex)) + J*log(n0),
-                   piecewiseSurvivalTime = u)
+      vtheta = 1/d*diag(1)
     }
 
+    fit2 <- list(model = "Piecewise exponential",
+                 theta = log(d/ex),
+                 vtheta = vtheta,
+                 bic = -2*sum(-d + d*log(d/ex)) + J*log(n0),
+                 piecewiseSurvivalTime = u)
+
     # fitted survival curve
     time = seq(0, max(df$time))
 
-    surv = 0
-    for (j in 1:J) {
-      exj = pmax(0, pmin(time, ucut[j+1]) - ucut[j])
-      surv = surv + exp(fit2$theta[j]) * exj
+    lambda = d/ex
+    if (J>1) {
+      psum = c(0, cumsum(lambda[1:(J-1)] * diff(u)))
+    } else {
+      psum = 0
     }
-    surv = exp(-surv)
+    j = findInterval(time, u)
+    m = psum[j] + lambda[j]*(time - u[j])
+    surv = exp(-m)
 
     dffit2 <- dplyr::tibble(time, surv)
   } else if (tolower(event_model) == "model averaging") {
